<?php

namespace Drupal\ics_media\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem;
use Drupal\ics\Plugin\Field\FieldFormatter\ImageComparisonSliderFormatter;
use Drupal\image\Plugin\Field\FieldType\ImageItem;
use Drupal\media\MediaInterface;

/**
 * Plugin implementation of the 'Image Comparison Slider' formatter for media.
 *
 * @FieldFormatter(
 *   id = "image_comparison_slider_media",
 *   label = @Translation("Image Comparison Slider"),
 *   field_types = {
 *     "entity_reference"
 *   }
 * )
 */
class ImageComparisonSliderMediaFormatter extends ImageComparisonSliderFormatter {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $settings = $this->getSettings();

    $style = $this->getImageStyle($settings);

    $options = $this->getOptions($settings);

    $images = [];

    foreach ($items as $item) {
      $media = $item->entity;
      $source_field_name = $media->getSource()
        ->getConfiguration()['source_field'];
      $image_item = $media->get($source_field_name)[0];
      // Experimental handling of media entities referencing other entities.
      // See: https://www.drupal.org/project/media_entity_reference.
      if (($image_item instanceof EntityReferenceItem)
        && !($image_item instanceof ImageItem)) {
        $target_entity = $image_item->entity;
        // We only handle referenced entities of type Media.
        if ($target_entity instanceof MediaInterface) {
          $target_source_field_name = $target_entity->getSource()
            ->getConfiguration()['source_field'];
          $image_item = $target_entity->get($target_source_field_name)[0];
        }
      }
      // We only handle medias of type Image.
      if ($image_item instanceof ImageItem) {
        $image = $this->getImageFromItem($image_item, $style);
        if (!is_null($image)) {
          if (isset($options['media_caption_field'])) {
            $this->setCaption($media, $image, $options['media_caption_field']);
          }
          $images[] = $image;
        }

        if (count($images) == 2) {
          break;
        }
      }
    }

    return $this->buildRenderArray($images, $options);
  }

  /**
   * {@inheritdoc}
   */
  public static function isApplicable(FieldDefinitionInterface $field_definition) {
    // This formatter is only available for entity types that reference
    // media items.
    return ($field_definition->getFieldStorageDefinition()
        ->getSetting('target_type') == 'media');
  }

  /**
   * @param \Drupal\media\MediaInterface $media
   * @param array $image
   * @param string $media_caption_field
   *
   * @return void
   */
  protected function setCaption(MediaInterface $media, array &$image, string $media_caption_field) {
    if ($media->hasField($media_caption_field)) {
      try {
        $caption = $media->get($media_caption_field)[0]->value;
        if (!empty($caption)) {
          $image['caption'] = $caption;
        }
      }
      catch (\Exception $e) {
        \Drupal::logger('ics')->warning($e->getMessage());
      }
    }
  }

}
